package com.jz.common.aspect;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import com.alibaba.fastjson.JSONObject;

/**方法日志打印
 * @author ybz
 *
 */
@Aspect
@Component
public class MethodCallLogPrintAspect {
	
	private static Logger logger = LoggerFactory.getLogger(MethodCallLogPrintAspect.class);
	
    @Pointcut("@annotation( com.jz.common.annotation.MethodCallLogPrint)")
    public void methodCallLogPrintPointCut(){

    }

    @Around("methodCallLogPrintPointCut()")
    public Object methodCallLogPrint(ProceedingJoinPoint pjp)throws Throwable{
        
        //从切面织入点处通过反射机制获取织入点处的方法
        MethodSignature signature = (MethodSignature) pjp.getSignature();
        //获取切入点所在的方法
        Method method = signature.getMethod();
        //获取请求的类名
        String className = pjp.getTarget().getClass().getName();
        //获取请求的方法名
        String methodName = method.getName();
        
        logger.info("**{}**{}**START****************************************", className, methodName);
        Long currentTime = System.currentTimeMillis();
        logger.info("开始时间：{}", currentTime);

        //请求的参数
        String[] argNames = signature.getParameterNames(); // 参数名
        Object[] argObjs = pjp.getArgs() ;
        Map<String, Object> paramMap = new HashMap<String, Object>();
        for (int index=0; index<argNames.length; index++) {
        	String argName = argNames[index];
        	Object argObj = argObjs[index];
        	if (argObj instanceof HttpServletRequest) {
				continue;
			}
			paramMap.put(argName, argObj);
		}
        
        logger.info("请求参数：【{}】", JSONObject.toJSONString(paramMap));
		Object obj = pjp.proceed();
		Long currentTime2 = System.currentTimeMillis();
		logger.info("调用结果：【{}】", JSONObject.toJSONString(obj));
        logger.info("方法调用结束");
        logger.info("当前时间：{}，总耗时：{}", currentTime2, currentTime2-currentTime);
        logger.info("**END****************************************");
        
        return obj;

    }
    
    
}