package com.jz.dmp.modules.service.projconfig.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.dmp.modules.controller.projconfig.bean.DmpProjectConfigEngineBatch;
import com.jz.dmp.modules.controller.projconfig.bean.DmpProjectConfigEngineDto;
import com.jz.dmp.modules.controller.projconfig.bean.DmpProjectConfigEngineRequest;
import com.jz.dmp.modules.dao.projconfig.DmpProjectConfigEngineMapper;
import com.jz.dmp.modules.model.DmpProjectConfigEngine;
import com.jz.dmp.modules.service.projconfig.DmpProjectConfigEngineService;

/**
 * 项目配置计算引擎关系表服务的实现?
 * 
 * @author ybz
 *
 */
@Service("dmpProjectConfigEngineService")
public class DmpProjectConfigEngineServiceImpl extends BaseService implements DmpProjectConfigEngineService {

	private static Logger logger = LoggerFactory.getLogger(DmpProjectConfigEngineServiceImpl.class);

	@Autowired
	private DmpProjectConfigEngineMapper dmpProjectConfigEngineMapper;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpProjectConfigEngineDto> findListWithPage(DmpProjectConfigEngineRequest dmpProjectConfigEngineRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpProjectConfigEngineDto> pageInfoResponse = new PageInfoResponse<>();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpProjectConfigEngineRequest.getConfigEngineId() != null) {
			param.put("configEngineId", dmpProjectConfigEngineRequest.getConfigEngineId());
		}
		//项目主键
		if (dmpProjectConfigEngineRequest.getProjectId() != null) {
			param.put("projectId", dmpProjectConfigEngineRequest.getProjectId());
		}
		//引擎主键
		if (dmpProjectConfigEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpProjectConfigEngineRequest.getEngineId());
		}
		//备注
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getRemark())) {
			param.put("remark", dmpProjectConfigEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpProjectConfigEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpProjectConfigEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpProjectConfigEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpProjectConfigEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpProjectConfigEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpProjectConfigEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpProjectConfigEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpProjectConfigEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpProjectConfigEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpProjectConfigEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpProjectConfigEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpProjectConfigEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpProjectConfigEngineRequest.getUpdateTimeEnd());
		}

		PageHelper.startPage(dmpProjectConfigEngineRequest.getPageNum(), dmpProjectConfigEngineRequest.getPageSize());
		List<DmpProjectConfigEngineDto> list = dmpProjectConfigEngineMapper.findList(param);
		PageInfo<DmpProjectConfigEngineDto> pageInfo = new PageInfo<>(list);

		pageInfoResponse.setCode(StatuConstant.SUCCESS_CODE);
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpProjectConfigEngineDto> findList(DmpProjectConfigEngineRequest dmpProjectConfigEngineRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpProjectConfigEngineDto> baseBeanResponse = new BaseBeanResponse<>();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpProjectConfigEngineRequest.getConfigEngineId() != null) {
			param.put("configEngineId", dmpProjectConfigEngineRequest.getConfigEngineId());
		}
		//项目主键
		if (dmpProjectConfigEngineRequest.getProjectId() != null) {
			param.put("projectId", dmpProjectConfigEngineRequest.getProjectId());
		}
		//引擎主键
		if (dmpProjectConfigEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpProjectConfigEngineRequest.getEngineId());
		}
		//备注
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getRemark())) {
			param.put("remark", dmpProjectConfigEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpProjectConfigEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpProjectConfigEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpProjectConfigEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpProjectConfigEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpProjectConfigEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpProjectConfigEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpProjectConfigEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpProjectConfigEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpProjectConfigEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpProjectConfigEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpProjectConfigEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpProjectConfigEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpProjectConfigEngineRequest.getUpdateTimeEnd());
		}

		List<DmpProjectConfigEngineDto> list = dmpProjectConfigEngineMapper.findList(param);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#add(com.ycxc.vmts.entity.DmpProjectConfigEngine,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpProjectConfigEngine> add(DmpProjectConfigEngine dmpProjectConfigEngine, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectConfigEngine> baseBeanResponse = new BaseBeanResponse<>();

		/*$*setConfigEngineIdCodeReplace*$*/
		// 创建人
		dmpProjectConfigEngine.setCreateUserId(getHttpRequestUserId(httpRequest));
		// 创建时间
		dmpProjectConfigEngine.setCreateTime(new Date());

		dmpProjectConfigEngineMapper.insertSelective(dmpProjectConfigEngine);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpProjectConfigEngine);

		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#edit(com.ycxc.vmts.entity.DmpProjectConfigEngine,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpProjectConfigEngine> edit(DmpProjectConfigEngine dmpProjectConfigEngine, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectConfigEngine> baseBeanResponse = new BaseBeanResponse<>();

		// 修改人
		dmpProjectConfigEngine.setUpdateUserId(getHttpRequestUserId(httpRequest));
		// 修改时间
		dmpProjectConfigEngine.setUpdateTime(new Date());

		dmpProjectConfigEngineMapper.updateByPrimaryKeySelective(dmpProjectConfigEngine);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpProjectConfigEngine);

		return baseBeanResponse;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#addOrEdit(com.ycxc.vmts.entity.DmpProjectConfigEngine,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpProjectConfigEngine> addOrEdit(DmpProjectConfigEngine dmpProjectConfigEngine, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpProjectConfigEngine> baseBeanResponse = new BaseBeanResponse<>();
		
		Integer configEngineId = dmpProjectConfigEngine.getConfigEngineId();
		
		if (null==configEngineId) {//新增
			// 创建人
			dmpProjectConfigEngine.setCreateUserId(getHttpRequestUserId(httpRequest));
			// 创建时间
			dmpProjectConfigEngine.setCreateTime(new Date());

			dmpProjectConfigEngineMapper.insertSelective(dmpProjectConfigEngine);

			baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
			baseBeanResponse.setMessage("新增成功");
			baseBeanResponse.setData(dmpProjectConfigEngine);
		} else {//修改
			// 修改人
			dmpProjectConfigEngine.setUpdateUserId(getHttpRequestUserId(httpRequest));
			// 修改时间
			dmpProjectConfigEngine.setUpdateTime(new Date());

			dmpProjectConfigEngineMapper.updateByPrimaryKeySelective(dmpProjectConfigEngine);

			baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
			baseBeanResponse.setMessage("修改成功");
			baseBeanResponse.setData(dmpProjectConfigEngine);
		}
		
		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#findById(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpProjectConfigEngineDto> findById(Integer configEngineId, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpProjectConfigEngineDto> baseBeanResponse = new BaseBeanResponse<>();

		DmpProjectConfigEngineDto dmpProjectConfigEngineDto = dmpProjectConfigEngineMapper.findById(configEngineId);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpProjectConfigEngineDto);

		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse deleteById(Integer configEngineId, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpProjectConfigEngineMapper.deleteByPrimaryKey(configEngineId);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse softDeleteById(Integer configEngineId, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpProjectConfigEngineMapper.softDeleteByPrimaryKey(configEngineId);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse delete(DmpProjectConfigEngineRequest dmpProjectConfigEngineRequest, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpProjectConfigEngineRequest.getConfigEngineId() != null) {
			param.put("configEngineId", dmpProjectConfigEngineRequest.getConfigEngineId());
		}
		//项目主键
		if (dmpProjectConfigEngineRequest.getProjectId() != null) {
			param.put("projectId", dmpProjectConfigEngineRequest.getProjectId());
		}
		//引擎主键
		if (dmpProjectConfigEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpProjectConfigEngineRequest.getEngineId());
		}
		//备注
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getRemark())) {
			param.put("remark", dmpProjectConfigEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpProjectConfigEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpProjectConfigEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpProjectConfigEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpProjectConfigEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpProjectConfigEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpProjectConfigEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpProjectConfigEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpProjectConfigEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpProjectConfigEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpProjectConfigEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpProjectConfigEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpProjectConfigEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpProjectConfigEngineRequest.getUpdateTimeEnd());
		}

		dmpProjectConfigEngineMapper.delete(param);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse softDelete(DmpProjectConfigEngineRequest dmpProjectConfigEngineRequest, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpProjectConfigEngineRequest.getConfigEngineId() != null) {
			param.put("configEngineId", dmpProjectConfigEngineRequest.getConfigEngineId());
		}
		//项目主键
		if (dmpProjectConfigEngineRequest.getProjectId() != null) {
			param.put("projectId", dmpProjectConfigEngineRequest.getProjectId());
		}
		//引擎主键
		if (dmpProjectConfigEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpProjectConfigEngineRequest.getEngineId());
		}
		//备注
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getRemark())) {
			param.put("remark", dmpProjectConfigEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpProjectConfigEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpProjectConfigEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpProjectConfigEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpProjectConfigEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpProjectConfigEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpProjectConfigEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpProjectConfigEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpProjectConfigEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpProjectConfigEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpProjectConfigEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpProjectConfigEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpProjectConfigEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpProjectConfigEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpProjectConfigEngineRequest.getUpdateTimeEnd());
		}

		dmpProjectConfigEngineMapper.softDelete(param);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpProjectConfigEngineService#addBatch(com.ycxc.vmts.controller.bas.
	 * bean.DmpProjectConfigEngineVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpProjectConfigEngineBatch> addBatch(DmpProjectConfigEngineBatch dmpProjectConfigEngineBatch, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpProjectConfigEngineBatch> baseBeanResponse = new BaseBeanResponse<>();

		List<DmpProjectConfigEngine> dmpProjectConfigEngines = dmpProjectConfigEngineBatch.getDmpProjectConfigEngines();

		if (CollectionUtils.isEmpty(dmpProjectConfigEngines)) {
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}

		dmpProjectConfigEngines = dmpProjectConfigEngines.stream().map(dmpProjectConfigEngine -> {
			/*$*setConfigEngineIdCodeReplace*$*/
			// 创建人
			try {
				dmpProjectConfigEngine.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			// 创建时间
			dmpProjectConfigEngine.setCreateTime(new Date());

			return dmpProjectConfigEngine;
		}).collect(Collectors.toList());

		dmpProjectConfigEngineMapper.insertBatch(dmpProjectConfigEngines);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpProjectConfigEngineBatch);

		return baseBeanResponse;
	}

	/**
	 *批量删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpProjectConfigEngineMapper.deleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/**
	 *批量软删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpProjectConfigEngineMapper.softDeleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

}
