package com.jz.dmp.modules.service.sys.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.Constants;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.modules.controller.sys.bean.DmpPermissionBatch;
import com.jz.dmp.modules.controller.sys.bean.DmpPermissionDto;
import com.jz.dmp.modules.controller.sys.bean.DmpPermissionRequest;
import com.jz.dmp.modules.controller.sys.bean.MyDmpPermissionConverter;
import com.jz.dmp.modules.dao.sys.DmpPermissionMapper;
import com.jz.dmp.modules.model.DmpMember;
import com.jz.dmp.modules.model.DmpPermission;
import com.jz.dmp.modules.model.DmpRole;
import com.jz.dmp.modules.service.sys.DmpPermissionService;

/**
 * 资源表服务的实现?
 * @author ybz
 *
 */
@Service("dmpPermissionService")
public class DmpPermissionServiceImpl extends BaseService implements DmpPermissionService {
	
	private static Logger logger = LoggerFactory.getLogger(DmpPermissionServiceImpl.class);
	
	@Autowired
	private DmpPermissionMapper dmpPermissionMapper;

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpPermissionService#findList(com.ycxc.vmts.controller.bean.DmpPermissionRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpPermissionDto> findList(DmpPermissionRequest dmpPermissionRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpPermissionDto> pageInfoResponse = new PageInfoResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//资源ID
		if (dmpPermissionRequest.getPermissionId() != null) {
			param.put("permissionId", dmpPermissionRequest.getPermissionId());
		}
		//资源编码
		if (!StringUtils.isEmpty(dmpPermissionRequest.getPermissionCode())) {
			param.put("permissionCode", dmpPermissionRequest.getPermissionCode());
		}
		//资源名称
		if (!StringUtils.isEmpty(dmpPermissionRequest.getPermissionName())) {
			param.put("permissionName", dmpPermissionRequest.getPermissionName());
		}
		//资源描述
		if (!StringUtils.isEmpty(dmpPermissionRequest.getPermissionDesc())) {
			param.put("permissionDesc", dmpPermissionRequest.getPermissionDesc());
		}
		//资源URL
		if (!StringUtils.isEmpty(dmpPermissionRequest.getPermissionUrl())) {
			param.put("permissionUrl", dmpPermissionRequest.getPermissionUrl());
		}
		//资源类型(关联字典编码表)
		if (!StringUtils.isEmpty(dmpPermissionRequest.getPermissionType())) {
			param.put("permissionType", dmpPermissionRequest.getPermissionType());
		}
		//排序
		if (dmpPermissionRequest.getPermissionSort() != null) {
			param.put("permissionSort", dmpPermissionRequest.getPermissionSort());
		}
		//父级编码
		if (!StringUtils.isEmpty(dmpPermissionRequest.getParentCode())) {
			param.put("parentCode", dmpPermissionRequest.getParentCode());
		}
		//备注
		if (!StringUtils.isEmpty(dmpPermissionRequest.getRemark())) {
			param.put("remark", dmpPermissionRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpPermissionRequest.getDataStatus())) {
			param.put("dataStatus", dmpPermissionRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpPermissionRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpPermissionRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpPermissionRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpPermissionRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpPermissionRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpPermissionRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpPermissionRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpPermissionRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpPermissionRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpPermissionRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpPermissionRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpPermissionRequest.getUpdateTimeEnd());
		}
		
		PageHelper.startPage(dmpPermissionRequest.getPageNum(), dmpPermissionRequest.getPageSize());
		List<DmpPermissionDto> list = dmpPermissionMapper.findList(param);
		PageInfo<DmpPermissionDto> pageInfo = new PageInfo<>(list);
		
		pageInfoResponse.setCode(ResultCode.SUCCESS.val());
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}
	

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpPermissionService#add(com.ycxc.vmts.entity.DmpPermission, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpPermission> add(DmpPermission dmpPermission, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpPermission> baseBeanResponse = new BaseBeanResponse<>();
		
		/*$*setPermissionIdCodeReplace*$*/
		//创建人
		dmpPermission.setCreateUserId(getHttpRequestUserId(httpRequest));
		//创建时间
		dmpPermission.setCreateTime(new Date());
		
		dmpPermissionMapper.insertSelective(dmpPermission);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpPermission);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpPermissionService#edit(com.ycxc.vmts.entity.DmpPermission, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpPermission> edit(DmpPermission dmpPermission, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpPermission> baseBeanResponse = new BaseBeanResponse<>();
		
		//修改人
		dmpPermission.setUpdateUserId(getHttpRequestUserId(httpRequest));
		//修改时间
		dmpPermission.setUpdateTime(new Date());
		
		dmpPermissionMapper.updateByPrimaryKeySelective(dmpPermission);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpPermission);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpPermissionService#findById(com.ycxc.vmts.controller.bean.DmpPermissionRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpPermissionDto> findById(DmpPermissionRequest dmpPermissionRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpPermissionDto> baseBeanResponse = new BaseBeanResponse<>();
		
		DmpPermissionDto dmpPermissionDto = dmpPermissionMapper.findById(dmpPermissionRequest.getPermissionId());
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpPermissionDto);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpPermissionService#deleteById(com.ycxc.vmts.controller.bean.DmpPermissionRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteById(DmpPermissionRequest dmpPermissionRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpPermissionMapper.deleteByPrimaryKey(dmpPermissionRequest.getPermissionId());
		
		baseResponse.setCode(ResultCode.SUCCESS.val());
		baseResponse.setMessage("删除成功");
		
		return baseResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpPermissionService#addBatch(com.ycxc.vmts.controller.bean.DmpPermissionVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpPermissionBatch> addBatch(DmpPermissionBatch dmpPermissionBatch, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpPermissionBatch> baseBeanResponse = new BaseBeanResponse<>();
		
		List<DmpPermission> dmpPermissions = dmpPermissionBatch.getDmpPermissions();
		
		if (CollectionUtils.isEmpty(dmpPermissions)) {
			baseBeanResponse.setCode(ResultCode.PARAMS_ERROR.val());
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}
		
		dmpPermissions = dmpPermissions.stream().map(dmpPermission -> {
			/*$*setPermissionIdCodeReplace*$*/
			//创建人
			try {
				dmpPermission.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			//创建时间
			dmpPermission.setCreateTime(new Date());
			
			return dmpPermission;
		}).collect(Collectors.toList());
		
		dmpPermissionMapper.insertBatch(dmpPermissions);
		
		baseBeanResponse.setCode(ResultCode.SUCCESS.val());
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpPermissionBatch);
		
		return baseBeanResponse;
	}


	/**
	 *根据角色获取所有的资源
	 */
	@Override
	public List<DmpPermission> findPermissionByRoles(List<DmpRole> dmpRoles) {
		List<DmpPermission> permissions = new ArrayList<DmpPermission>();
		
		try {
			List<DmpPermission> dmpPermissions = dmpPermissionMapper.findByRoles(dmpRoles);
			String parentCode = "0";
			//整理资源为树形结构
			permissions = permissionArrangeTree(parentCode, dmpPermissions);
		} catch (Exception e) {
			logger.info("根据角色查询资源异常：{}", e);
		}
		
		return permissions;
	}
	
	/**  
	* @Title: permissionArrangeTree  
	* @Description: TODO(将所有的资源整理成树形结构)  
	* @param @param dmpPermissions
	* @param @return
	* @param @throws Exception    参数  
	* @return List<DmpPermission>    返回类型  
	* @throws  
	*/ 
	private List<DmpPermission> permissionArrangeTree(String parentCode,List<DmpPermission> dmpPermissions)throws Exception{
		
		Map<String, List<DmpPermission>> dictMap = dmpPermissions.stream().collect(Collectors.groupingBy(DmpPermission::getParentCode));
		List<DmpPermission> permissionTree = dictMap.get(parentCode);//获取顶层资源
		
		if (!CollectionUtils.isEmpty(permissionTree)) {
			for (DmpPermission dmpPermission : permissionTree) {
				dmpPermission = arrangeChildren(dmpPermission, dictMap);
			}
		}
		
		return permissionTree;
	}
	
	/**  
	* @Title: arrangeChildren  
	* @Description: TODO(这里用一句话描述这个方法的作用)  
	* @param @param permission
	* @param @param dictMap
	* @param @return
	* @param @throws Exception    参数  
	* @return DmpPermission    返回类型  
	* @throws  
	*/ 
	private DmpPermission arrangeChildren(DmpPermission permission, Map<String, List<DmpPermission>> dictMap)throws Exception{
		
		String permissionCode = permission.getPermissionCode();
		List<DmpPermission> children = dictMap.get(permissionCode); 
		
		if (!CollectionUtils.isEmpty(children)) {
			for (DmpPermission dmpPermission : children) {
				dmpPermission = arrangeChildren(dmpPermission, dictMap);
			}
			permission.setChildren(children);
		}
		
		return permission;
	}


	/**
	 *获取所有资源
	 */
	@Override
	public List<DmpPermission> findAllList() {
		
		List<DmpPermission> permissions = new ArrayList<DmpPermission>();
		try {
			List<DmpPermissionDto> permissionDtos = dmpPermissionMapper.findAllListWithUrlNotNull();
			permissions = MyDmpPermissionConverter.INSTANCE().dto2domain(permissionDtos);
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.info("获取所有资源接口异常：{}", e);
		}
		
		
		return permissions;
	}


	/**
	 *获取父资源编码下当前用户项目所有资源
	 */
	@Override
	public BaseBeanResponse<DmpPermissionDto> findListByParentCode(String parentCode,
			String type, HttpServletRequest httpRequest) throws Exception {
		
		BaseBeanResponse<DmpPermissionDto> baseBeanResponse = new BaseBeanResponse<>();
		
		//获取当前用户拥有的项目角色
		DmpMember currentUser = SessionUtils.getSecurityUser();
		if (currentUser==null) {
			baseBeanResponse.setCode(StatuConstant.CODE_NOT_SIGNED);
			baseBeanResponse.setMessage("用户未登录！");
			return baseBeanResponse;
		}
		
		List<DmpPermission> dmpPermissions = new ArrayList<DmpPermission>();
		if ("0".equals(type)) {
			dmpPermissions = dmpPermissionMapper.findByRoles(currentUser.getMemberProjectRoleList());
		}else if("1".equals(type)) {
			dmpPermissions = dmpPermissionMapper.findByRoles(currentUser.getMemberRoleList());
		}else {
			List<DmpPermission> list1 = dmpPermissionMapper.findByRoles(currentUser.getMemberProjectRoleList());
			List<DmpPermission> list2 = dmpPermissionMapper.findByRoles(currentUser.getMemberRoleList());
			List<DmpPermission> permissions = new ArrayList<DmpPermission>();
			if (!CollectionUtils.isEmpty(list1)) {
				permissions.addAll(list1);
			}
			if (!CollectionUtils.isEmpty(list2)) {
				permissions.addAll(list2);
			}
			dmpPermissions = permissions;
		}
		
		//整理资源为树形结构
		List<DmpPermission> permissions = permissionArrangeTree(parentCode, dmpPermissions);
		
		List<DmpPermissionDto> list = MyDmpPermissionConverter.INSTANCE().domain2dto(permissions);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		
		return baseBeanResponse;
	}
	
	/**
	 *获取父资源编码下当前用户项目所有资源
	 */
	@Override
	public BaseBeanResponse<Map<String, Object>> findUserAndPremissionByParentCode(String parentCode,
			String type, HttpServletRequest httpRequest) throws Exception {
		
		Map<String, Object> resultMap = new HashMap<String, Object>();
		
		BaseBeanResponse<Map<String, Object>> baseBeanResponse = new BaseBeanResponse<>();
		
		//获取当前用户拥有的项目角色
		DmpMember currentUser = SessionUtils.getSecurityUser();
		if (currentUser==null) {
			baseBeanResponse.setCode(StatuConstant.CODE_NOT_SIGNED);
			baseBeanResponse.setMessage("用户未登录！");
			return baseBeanResponse;
		}
		
		resultMap.put("user", currentUser);
		
		List<DmpPermission> dmpPermissions = new ArrayList<DmpPermission>();
		if ("0".equals(type)) {
			dmpPermissions = dmpPermissionMapper.findByRoles(currentUser.getMemberProjectRoleList());
		}else if("1".equals(type)) {
			dmpPermissions = dmpPermissionMapper.findByRoles(currentUser.getMemberRoleList());
		}else {
			List<DmpPermission> list1 = dmpPermissionMapper.findByRoles(currentUser.getMemberProjectRoleList());
			List<DmpPermission> list2 = dmpPermissionMapper.findByRoles(currentUser.getMemberRoleList());
			List<DmpPermission> permissions = new ArrayList<DmpPermission>();
			if (!CollectionUtils.isEmpty(list1)) {
				permissions.addAll(list1);
			}
			if (!CollectionUtils.isEmpty(list2)) {
				permissions.addAll(list2);
			}
			dmpPermissions = permissions;
		}
		
		//整理资源为树形结构
		List<DmpPermission> permissions = permissionArrangeTree(parentCode, dmpPermissions);
		
		List<DmpPermissionDto> list = MyDmpPermissionConverter.INSTANCE().domain2dto(permissions);
		
		resultMap.put("permissions", list);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(resultMap);
		
		return baseBeanResponse;
	}
	
}
